`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    18:42:21 03/18/2008 
// Design Name: 
// Module Name:    BRAM_shift 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
/*
This module takes the islocal_min matrix generated and creates a sorted output of
local min coordinates for input into the BRAM storage for clustering.
*/
module BRAM_shift(	
							islocal_min,							
							//Since the row and column numbers for each block (IN PIXELS) is reused later (in BRAM_shift.v, among
							//others,) it is more prudent to give block_addr_column and block_addr_row
							pointn,				//I [8:0]		//rows
							pointm,				//I [9:0]		//columns
								
							source_addr,		//O [8:0]		sequential out for BRAM	
							source_data,		//O
							source_clk,			//O
							source_en,			//O
							source_wen,			//O
							
							clk,
							go,
							rst,
							done
						);

parameter ISLOCAL_MIN_BITS = 24;

input		clk, go, rst;
output 	done;
	
input 	[ISLOCAL_MIN_BITS-1:0] 		islocal_min;

input 	[8:0]	  							pointn;
input 	[9:0]	   						pointm;

output	[8:0]								source_addr;
output	[31:0]							source_data;
output										source_clk;
output										source_en;
output										source_wen;				
				
//registers
	//register the islocal_min input to allow for pipelining
	reg 	[ISLOCAL_MIN_BITS-1:0] 		islocal_min_reg;	
	//state register
	reg [2:0] state;
	//BRAM write address
	reg [8:0] state_address;
	//done reg
	reg done;

//wires for state machine
	wire SR_en;
	wire i_reset;

//last output wire
	wire [18:0]	SR_out;


//BRAM control signal assignments
assign source_data = {13'b0,SR_out};
assign source_addr = state_address;
assign source_clk	 = clk;
assign source_en = 1'b1;



/*
DATA FORMAT
First nine bits are the n coordinate (rows.) Next ten bits are the m
coordinate (columns.) These are stored little endian.
32-bit register
0      78 9     15 18    23       31
|------|| ------||------| |------|
[   n   ] [    m   ] 
LSB   MSB LSB	  MSB 
*/

//State machine
always@( posedge clk ) begin
	if( rst ) begin
		state 				= 0;
		state_address 		= 0;
		done 					= 0;
		islocal_min_reg 	= 0;
	end
	else begin
		case( state )
			0: begin		//wait state
				if( go ) begin
					state = 1;
					islocal_min_reg = islocal_min;	//register inputs - SR operates off these registered inputs, set at the beginning of every operation
				end
			end
			
			1: begin		//delay for SR to get set
				state = 5;
			end
			
			2: begin		//check state
				if( source_data == 0 ) begin	//done, finish up.
					state = 6;
					done = 1;		//pulse done signal -- NOT YET TESTED!
				end
				else
					state = 3;
			end
			
			3: begin
				state = 4;
			end
			
			4: begin
				state_address = state_address + 1;
				state = 2;
			end
			
			5: begin		//delay cycle 2, for a total of 20ns, just to be safe.
				state = 2;
			end
			6: begin
				done = 0;
				state = 0;
			end
			default:
				state = 0;	
		endcase
	end		//else
end		//always

//output assignments
assign source_wen 	= (state == 3);
assign SR_en 			= (state == 4);
assign i_reset			= (state == 5);


////////////////////////////////////////////////////////////////
/*Fancy shift register! See the notes for details on how it operates.
The basics are like this: The shift register is initialized at the beginning
of every run with the pixel addresses of every pixel in the current block.
If the islocal_min[n] is low, the register is bypassed in the SR. This
results in an SR that contains only the elements that are local minima. Once
this procedure is completed, the contents of the SR is clocked into the BRAM used to perform
clustering.
*/

//Create registers for shifter
reg  [18:0]  point_00;
reg  [18:0]  point_01;
reg  [18:0]  point_02;
reg  [18:0]  point_03;
reg  [18:0]  point_04;
reg  [18:0]  point_05;
reg  [18:0]  point_06;
reg  [18:0]  point_07;
reg  [18:0]  point_08;
reg  [18:0]  point_09;
reg  [18:0]  point_10;
reg  [18:0]  point_11;
reg  [18:0]  point_12;
reg  [18:0]  point_13;
reg  [18:0]  point_14;
reg  [18:0]  point_15;
reg  [18:0]  point_16;
reg  [18:0]  point_17;
reg  [18:0]  point_18;
reg  [18:0]  point_19;
reg  [18:0]  point_20;
reg  [18:0]  point_21;
reg  [18:0]  point_22;
reg  [18:0]  point_23;

//wire assignments for shift register
wire [18:0]  point_00_d;
wire [18:0]  point_01_d;
wire [18:0]  point_02_d;
wire [18:0]  point_03_d;
wire [18:0]  point_04_d;
wire [18:0]  point_05_d;
wire [18:0]  point_06_d;
wire [18:0]  point_07_d;
wire [18:0]  point_08_d;
wire [18:0]  point_09_d;
wire [18:0]  point_10_d;
wire [18:0]  point_11_d;
wire [18:0]  point_12_d;
wire [18:0]  point_13_d;
wire [18:0]  point_14_d;
wire [18:0]  point_15_d;
wire [18:0]  point_16_d;
wire [18:0]  point_17_d;
wire [18:0]  point_18_d;
wire [18:0]  point_19_d;
wire [18:0]  point_20_d;
wire [18:0]  point_21_d;
wire [18:0]  point_22_d;
wire [18:0]  point_23_d;

wire [18:0]  point_00_q;
wire [18:0]  point_01_q;
wire [18:0]  point_02_q;
wire [18:0]  point_03_q;
wire [18:0]  point_04_q;
wire [18:0]  point_05_q;
wire [18:0]  point_06_q;
wire [18:0]  point_07_q;
wire [18:0]  point_08_q;
wire [18:0]  point_09_q;
wire [18:0]  point_10_q;
wire [18:0]  point_11_q;
wire [18:0]  point_12_q;
wire [18:0]  point_13_q;
wire [18:0]  point_14_q;
wire [18:0]  point_15_q;
wire [18:0]  point_16_q;
wire [18:0]  point_17_q;
wire [18:0]  point_18_q;
wire [18:0]  point_19_q;
wire [18:0]  point_20_q;
wire [18:0]  point_21_q;
wire [18:0]  point_22_q;
wire [18:0]  point_23_q;


assign point_00_d = 19'b0;
assign point_00_q = point_00;
assign point_01_d = islocal_min_reg[0] ? point_00_q : point_00_d;
assign point_01_q = point_01;
assign point_02_d = islocal_min_reg[1] ? point_01_q : point_01_d;
assign point_02_q = point_02;
assign point_03_d = islocal_min_reg[2] ? point_02_q : point_02_d;
assign point_03_q = point_03;
assign point_04_d = islocal_min_reg[3] ? point_03_q : point_03_d;
assign point_04_q = point_04;
assign point_05_d = islocal_min_reg[4] ? point_04_q : point_04_d;
assign point_05_q = point_05;
assign point_06_d = islocal_min_reg[5] ? point_05_q : point_05_d;
assign point_06_q = point_06;
assign point_07_d = islocal_min_reg[6] ? point_06_q : point_06_d;
assign point_07_q = point_07;
assign point_08_d = islocal_min_reg[7] ? point_07_q : point_07_d;
assign point_08_q = point_08;
assign point_09_d = islocal_min_reg[8] ? point_08_q : point_08_d;
assign point_09_q = point_09;
assign point_10_d = islocal_min_reg[9] ? point_09_q : point_09_d;
assign point_10_q = point_10;
assign point_11_d = islocal_min_reg[10] ? point_10_q : point_10_d;
assign point_11_q = point_11;
assign point_12_d = islocal_min_reg[11] ? point_11_q : point_11_d;
assign point_12_q = point_12;
assign point_13_d = islocal_min_reg[12] ? point_12_q : point_12_d;
assign point_13_q = point_13;
assign point_14_d = islocal_min_reg[13] ? point_13_q : point_13_d;
assign point_14_q = point_14;
assign point_15_d = islocal_min_reg[14] ? point_14_q : point_14_d;
assign point_15_q = point_15;
assign point_16_d = islocal_min_reg[15] ? point_15_q : point_15_d;
assign point_16_q = point_16;
assign point_17_d = islocal_min_reg[16] ? point_16_q : point_16_d;
assign point_17_q = point_17;
assign point_18_d = islocal_min_reg[17] ? point_17_q : point_17_d;
assign point_18_q = point_18;
assign point_19_d = islocal_min_reg[18] ? point_18_q : point_18_d;
assign point_19_q = point_19;
assign point_20_d = islocal_min_reg[19] ? point_19_q : point_19_d;
assign point_20_q = point_20;
assign point_21_d = islocal_min_reg[20] ? point_20_q : point_20_d;
assign point_21_q = point_21;
assign point_22_d = islocal_min_reg[21] ? point_21_q : point_21_d;
assign point_22_q = point_22;
assign point_23_d = islocal_min_reg[22] ? point_22_q : point_22_d;
assign point_23_q = point_23;


assign SR_out = islocal_min_reg[23] ? point_23_q : 19'b0;

wire [8:0]  pointn;
wire [9:0]  pointm;

wire [8:0] 	pointn1 		= pointn + 1;
wire [8:0] 	pointn2 		= pointn + 2;
wire [8:0] 	pointn3 		= pointn + 3;

wire [9:0] 	pointm1 		= pointm + 1;
wire [9:0] 	pointm2 		= pointm + 2;
wire [9:0] 	pointm3 		= pointm + 3;
wire [9:0] 	pointm4 		= pointm + 4;
wire [9:0] 	pointm5 		= pointm + 5;

//logic for shift register
always@( posedge clk )
	if( i_reset ) begin		//initialize
		point_00 <= { (pointm),	 (pointn) };
		point_01 <= { (pointm1), (pointn) };
		point_02 <= { (pointm2), (pointn) };
		point_03 <= { (pointm3), (pointn) };
		point_04 <= { (pointm4), (pointn) };
		point_05 <= { (pointm5), (pointn) };
		point_06 <= { (pointm),  (pointn1) };
		point_07 <= { (pointm1), (pointn1) };
		point_08 <= { (pointm2), (pointn1) };
		point_09 <= { (pointm3), (pointn1) };
		point_10 <= { (pointm4), (pointn1) };
		point_11 <= { (pointm5), (pointn1) };
		point_12 <= { (pointm),  (pointn2) };
		point_13 <= { (pointm1), (pointn2) };
		point_14 <= { (pointm2), (pointn2) };
		point_15 <= { (pointm3), (pointn2) };
		point_16 <= { (pointm4), (pointn2) };
		point_17 <= { (pointm5), (pointn2) };
		point_18 <= { (pointm),  (pointn3) };
		point_19 <= { (pointm1), (pointn3) };
		point_20 <= { (pointm2), (pointn3) };
		point_21 <= { (pointm3), (pointn3) };
		point_22 <= { (pointm4), (pointn3) };
		point_23 <= { (pointm5), (pointn3) };
	end
	else if( SR_en ) begin
		point_00 <= point_00_d;
		point_01 <= point_01_d;
		point_02 <= point_02_d;
		point_03 <= point_03_d;
		point_04 <= point_04_d;
		point_05 <= point_05_d;
		point_06 <= point_06_d;
		point_07 <= point_07_d;
		point_08 <= point_08_d;
		point_09 <= point_09_d;
		point_10 <= point_10_d;
		point_11 <= point_11_d;
		point_12 <= point_12_d;
		point_13 <= point_13_d;
		point_14 <= point_14_d;
		point_15 <= point_15_d;
		point_16 <= point_16_d;
		point_17 <= point_17_d;
		point_18 <= point_18_d;
		point_19 <= point_19_d;
		point_20 <= point_20_d;
		point_21 <= point_21_d;
		point_22 <= point_22_d;
		point_23 <= point_23_d;
	end	


endmodule


