//******************************************************************************
//vga			-	VGA driver library
//
//By			-	Yi Yao				994085583
//Last revision	-	2008-03-04
//
//Notes:
//	In the plb_tft_cntlr_ref_v1_00_d pcore, only the 11 MSB of the frame buffer
//		base address are kept. This means that each frame has an address space
//		of 21 LSB = 2MB. The beginning of each frame must align on a 2MB
//		boundary. This is an implementation detail which should be abstracted
//		and kept hidden by this library.
//	Lines start every 1024 bytes. Even though the horizontal resolution is only
//		640, the extra 384 bytes are wasted for the sake of lines aligning to
//		kilobyte boundaries.
//	To use this library, you may want to:
//		1 - Instantiate a plb_tft_cntlr_ref_v1_00_d on the PLB in your system
//		2 - Add vga.h and vga.c to your sources
//		3 - Declare an offscreen buffer aligned to a 2MB boundary such as:
//				VGA_Buffer	MyBuffer = (VGA_Buffer) 0x00200000;
//		4 - Clear the off screen buffer and fill it up to your heart's content:
//				VGA_Clear(MyBuffer);
//				VGA_PSet(MyBuffer, 10, 10, VGA_COLOUR_RED);
//				etc
//		5 - Activate the offscreen buffer and turn on the display:
//				VGA_SetActiveBuffer(MyBuffer);
//				VGA_On();
//		6 - Write offscreen displays and set them active when you need them,
//				there is no need to turn on the display for all subsequent
//				frames
//******************************************************************************

#ifndef __VGA_H
#define __VGA_H

#include <xbasic_types.h>


//**************************************
//Custom datatypes

#define	VGA_Buffer	Xuint32 *
#define	VGA_Colour	Xuint32

//End of custom datatypes
//**************************************


//**************************************
//Public constants and global variables

#define	VGA_RES_X	640					//Horizontal resolution
#define	VGA_RES_Y	480					//Vertical resolution

#define VGA_COLOUR_WHITE	0x00FFFFFF
#define VGA_COLOUR_BLACK	0x00000000
#define VGA_COLOUR_RED		0x00FF0000
#define VGA_COLOUR_GREEN	0x0000FF00
#define VGA_COLOUR_BLUE		0x000000FF
#define VGA_COLOUR_YELLOW	0x00FFFF00
#define VGA_COLOUR_CYAN		0x0000FFFF
#define VGA_COLOUR_MAGENTA	0x00FF00FF
#define VGA_COLOUR_GREY		0x007F7F7F

extern int	VGA_VOffset;				//To compensate for frames starting midscreen

//End of public constants and global variables
//**************************************


//**************************************
//Private functions

//Address offset from start of frame given x and y coordinates
//C style declaration:
//	int	VGA_FOS(int x, int y);
#define VGA_FOS(x, y)	((((y + VGA_VOffset) % VGA_RES_Y)<< 10) + x)

//End of private functions
//**************************************


//**************************************
//Public functions


//The following are IO operations on the VGA core

//Sets the active buffer for display
void	VGA_SetActiveBuffer(VGA_Buffer buffer);

//Enables VGA output
void	VGA_On(void);

//Disables VGA output
void	VGA_Off(void);


//The follwing functions perform drawing features on buffers

//Clears a display buffer to black
void	VGA_Clear(VGA_Buffer buffer);

//Clears a display buffer to a specified colour
void	VGA_ClearC(VGA_Buffer buffer, VGA_Colour c);

//Sets a pixel
//C style declaration:
//	void	VGA_PSet(VGA_Buffer buffer, int x, int y, VGA_Colour c);
#define VGA_PSet(Buffer, x, y, c)	(Buffer[VGA_FOS(x, y)] = c)

//Returns the colour of a pixel
//C style declaration:
//	VGA_Colour	VGA_PGet(VGA_Buffer buffer, int x, int y);
#define	VGA_PGet(Buffer, x, y)		(Buffer[VGA_FOS(x, y)]);


//The following functions are helper functions to the VGA library

//Returns an abstracted colour type based on individual RGB channel intensities
//C style declaration:
//	VGA_Colour VGA_RGB2C(Xuint8 r, Xuint8 g, Xuint8 b);
#define	VGA_RGB2C(r, g, b)	((r << 16) | (g << 8) | b)

//End of public functions
//**************************************

#endif
